#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>

#include "rawvideoclip.h"
#include <asm/types.h>

#ifdef HAVE_CONFIG_H
#include <config.h>  // config.h by configure
#endif

#ifdef USE_KERNEL_VIDEODEV
#include <linux/videodev.h>
#else
#include <videodev.h>
#endif

static char graberror[256];
#define GRAB_PERROR(action,file) \
  {sprintf(graberror,"%s: %s: %s",action,file,strerror(errno));fprintf(stderr,graberror);return -1;}

#define TRAP(txt) fprintf(stderr,"%s:%d:%s\n",__FILE__,__LINE__,txt);exit(1);


/* ----------------------------------------------------------------------- */

rawVideoClip::rawVideoClip():videoClip()
{
  strcpy(ext,".raw");
  strcpy(name,"RAW video file");
  video_formats=VIDEO_PALETTE_RGB555 | VIDEO_PALETTE_RGB565 | VIDEO_PALETTE_RGB32 | VIDEO_PALETTE_YUV411;
  audio_formats=AUDIO_NONE;
  
}

int rawVideoClip::open(char *filename, struct MOVIE_PARAMS *par)
{
  frames=0;
  strcpy(file,filename);
  if (strstr(file,ext) == NULL) strcat(file,ext);
  strcpy(par->extension,ext);
  
  memcpy(&params,par,sizeof(struct MOVIE_PARAMS));
    switch (params.video_format) {
#if 0
  case VIDEO_RGB08:
    frame_bytes  = params.width * params.height;
    break;
#endif
  case VIDEO_PALETTE_RGB555:
    // this is BT848_COLOR_FMT_RGB15, best choice for high frame rates!!
  case VIDEO_PALETTE_RGB565:
    frame_bytes  = params.width * params.height *2;
    break;
#if 0
  case VIDEO_PALETTE_RGB24:
    frame_bytes  = params.width * params.height *3;
    break;
#endif
  case VIDEO_PALETTE_RGB32:
    frame_bytes  = params.width * params.height *3;
    break;
  case VIDEO_PALETTE_YUV411:  // doesnt work ...
    // this is BT848_COLOR_FMT_BtYUV, the best choice for high picture quality!!
    frame_bytes  = params.width * params.height *3/2;
    break;
  default: // VIDEO_RGB15
    frame_bytes  = params.width * params.height *2;
    break;
    //TRAP("unsupported video format");
  }
  
  if (-1 == (fd = ::open(filename,O_CREAT | O_RDWR | O_TRUNC, 0666)))
    GRAB_PERROR("open",filename);

  return 0;
}

int rawVideoClip::writeframe(void *data)
{
    if (-1 == write(fd,data,frame_bytes))
	GRAB_PERROR("write",file);
    frames++;
    return 0;
}

int rawVideoClip::close()
{
    ::close(fd);
    return 0;
}

