/* 
    xWinTV using Bt848 frame grabber driver

    Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <wintv.h>

#include <wintvchannel.h>

#include <kwm.h>
#include <kiconloader.h>
#include <klocale.h>

#define BID_ADD     0
#define BID_EDIT    1
#define BID_DELETE  2
#define BID_SL      3
#define BID_SC      4
#define BID_COPY    5
#define BID_PASTE   6 

#ifdef myDEBUG
#define chanDEBUG
#endif



/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/


const char* myQListViewItem::key( int column, bool order) const 
{
  if ( ! text( column ) ) return NULL;
  if ( column == -1 ) {
    int id=0;
    id= atoi( text(0) ) + 33;
    QString tmpString;
    tmpString.sprintf( "%c", id );
#ifdef chanDEBUG
    debug("key: %d",id);
#endif
    return tmpString;
  } else {
    return text(column);
  }
}

/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

winTVChannelWidget::winTVChannelWidget(QWidget* parent, const char* name, 
				       rcfile *_rc):  KTopLevelWidget( name ),rc(_rc)
{
#ifdef chanDEBUG
  debug("channel: begin of init");
#endif

  KWM::setIcon(winId(), kapp->getIcon());
  KWM::setMiniIcon(winId(), kapp->getMiniIcon());

  setCaption((const char*)(QString("program box")));
  // set icon

  KIconLoader *loader = kapp->getIconLoader();

  pm = loader->loadIcon("kwintv_xwintv.xpm");
  setIcon(pm);
  setIconText("kWinTV - program box");

  // toolbar
  toolbar = new KToolBar(this,"kwintv -- ToolBar --",26);
  toolbar->setBarPos(KToolBar::Top);
  addToolBar(toolbar);

  ch_add_xpm     = loader->loadIcon("kwintv_ch_add.xpm");
  ch_edit_xpm    = loader->loadIcon("kwintv_ch_edit.xpm");
  ch_delete_xpm  = loader->loadIcon("kwintv_ch_delete.xpm");
  sa_xpm         = loader->loadIcon("kwintv_sa.xpm");
  cs_xpm         = loader->loadIcon("kwintv_cs.xpm");
  
  toolbar->insertButton(ch_add_xpm,BID_ADD,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("add new program"));
  toolbar->insertButton(ch_edit_xpm,BID_EDIT,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("edit selected program"));
  toolbar->insertButton(ch_delete_xpm,BID_DELETE,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("cut selected program"));
#if 0
  toolbar->insertButton(ch_delete_xpm,BID_COPY,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("copy selected program"));
  toolbar->insertButton(ch_delete_xpm,BID_PASTE,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("paste selected program"));
#endif
  toolbar->insertSeparator();

  toolbar->insertButton(sa_xpm,BID_SL,SIGNAL(toggled(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("Stereo Mixer en/disable"));
  toolbar->setToggle(BID_SL);
  toolbar->insertSeparator();

  toolbar->insertButton(cs_xpm,BID_SC,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("hide program box"));

  channelbox= new winWidget(this,"program box and mixer",rc->sliders_att_startup);
  rc->sliders_att=rc->sliders_att_startup;
  setView(channelbox);

  connect(channelbox,SIGNAL(highlighted(int)),this,SLOT(channelSelectedEmitSLOT(int)));
  connect(channelbox,SIGNAL(valueChanged(int,int)),this,SLOT(volumeChangedEmitSLOT(int, int)));

  connect(channelbox,SIGNAL(moveItemSIGNAL( int, int )),this,SLOT(moveItemSLOT( int, int )));

  setMinimumSize( 180,100 );
  resize( 240,230 );
}

/* ------------------------------------------------------------------------------*/

winTVChannelWidget::~winTVChannelWidget()
{
#ifdef chanDEBUG
  debug("channel: destroying winTVChannelWidget");
#endif

  delete toolbar;
  delete channelbox;

#ifdef chanDEBUG
  debug("channel: winTVChannelWidget destroyed");
#endif
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::newChannelListSLOT( QList<channel> *clist )
{
  if ( clist->isEmpty() ) return;

  clistp= clist;

  int id= clist->at();

  inserting=TRUE;
  channelbox->clear();
  
  channel *c= clist->first();
  while ( c ) {
    c->id=clist->at();
    channelbox->insertItem( c, inserting );
#ifdef chanDEBUG
    //debug("channel: insertion %d, %s, %s",c->id,c->name,c->channelId);
#endif
    c= clist->next();
  }
  inserting=FALSE;

  clist->at(id);

#ifdef chanDEBUG
  debug("channel: channel list insertion done %d, %d",clist->count(),clist->at());
#endif
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::moveItemSLOT(int from, int to)
{
  inserting=TRUE;

  //clistp->setAutoDelete ( !clistp->autoDelete ()  );

  channel *cfrom= clistp->at(from);
  channel *cto= clistp->at(to);

  clistp->remove( clistp->find(cfrom) );
  clistp->insert( clistp->find(cto), cfrom );

  //clistp->setAutoDelete ( !clistp->autoDelete ()  );

  inserting=FALSE;

  newChannelListSLOT( clistp );
#ifdef chanDEBUG
  debug("channel: from %d",clistp->find(cfrom));
#endif
  channelbox->setCurrentItem( clistp->at() );
  channelbox->centerCurrentItem();
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::activeChannelChangedSLOT()
{
  if (inserting) return;

#ifdef chanDEBUG
  debug("channel: activeChannelChangedSLOT");
#endif

  channelbox->setCurrentItem( clistp->at() );
  channelbox->centerCurrentItem();
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::volumeChangedSLOT(int vl,int vr)
{
  channelbox->setValue(vl,vr);
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::toggleSplitSlidersSLOT()
{
  rc->sliders_att=!rc->sliders_att;
  channelbox->splitSliderSLOT(rc->sliders_att);
  emit toggleSplitSlidersSIGNAL();
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::toggleVisibilitySLOT( QPoint *to )
{
  if (isVisible()) hide();
  else {
    if ( to ) {
#ifdef chanDEBUG
      debug("channel: setGeometry %d,%d,%d,%d",to->x(), to->y(), width(), height());
#endif
      setGeometry( to->x(), to->y(), width(), height() );
    }
    show();
  }
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::volumeChangedEmitSLOT(int vl, int vr)
{
  emit volumeChangedSIGNAL(vl,vr);
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::channelSelectedEmitSLOT(int index)
{
  if (inserting) return;

#ifdef chanDEBUG
  debug("channel: channelSelectedEmitSLOT");
#endif

  if ( index != clistp->at() ) {
    clistp->at( index );
    emit channelSelectedSIGNAL();
  }
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::buttonsSLOT(int button)
{
  switch (button) {
  case BID_SC:
    toggleVisibilitySLOT( NULL );
    break;
  case BID_SL:
    toggleSplitSlidersSLOT();
    break;
  case BID_ADD:
    emit addChannelSIGNAL();
    break;
  case BID_EDIT:
    emit editChannelSIGNAL();
    break;
  case BID_DELETE:
    emit deleteChannelSIGNAL();
    break;
  case BID_COPY:
    emit copyChannelSIGNAL();
    break;
  case BID_PASTE:
    emit pasteChannelSIGNAL();
    break;
  default:
    // this should never happen...
    fatal(i18n("main: nonexisting button number"));
  }
}


/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::setPopups(QPopupMenu *sp,QPopupMenu *cp)
{
 channelbox->setSPopup(sp);
 channelbox->setCPopup(cp);
}

/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

#define Inherited QWidget

winWidget::winWidget(QWidget* parent,
		   const char* name,
		   int _sliderAtt)
  :sliderAtt(_sliderAtt),
   Inherited( parent, name, 503808 )
{
#ifdef chanDEBUG
  debug("win: init start");
#endif
  KIconLoader *loader = kapp->getIconLoader();
  ls_xpm = loader->loadIcon("mute.xpm");

  channelList = new winListBox( this );
  channelList->setGeometry( 5, 5, 140, 148 );
  channelList->setMinimumSize( 10, 10 );
  channelList->setMaximumSize( 32767, 32767 );
  /*
  {
    QColorGroup normal( QColor( QRgb(0) ), QColor( QRgb(12632256) ), QColor( QRgb(16777215) ), QColor( QRgb(6316128) ), QColor( QRgb(10789024) ), QColor( QRgb(32896) ), QColor( QRgb(16777215) ) );
    QColorGroup disabled( QColor( QRgb(8421504) ), QColor( QRgb(12632256) ), QColor( QRgb(16777215) ), QColor( QRgb(6316128) ), QColor( QRgb(10789024) ), QColor( QRgb(8421504) ), QColor( QRgb(12632256) ) );
    QColorGroup active( QColor( QRgb(8388608) ), QColor( QRgb(12632256) ), QColor( QRgb(16777215) ), QColor( QRgb(6316128) ), QColor( QRgb(10789024) ), QColor( QRgb(32896) ), QColor( QRgb(16777215) ) );
    QPalette palette( normal, disabled, active );
    channelList->setPalette( palette );
  }
  */
  {
    QFont font( "helvetica", 14, 20, 0 ); // 63
    font.setStyleHint( (QFont::StyleHint)0 );
    font.setCharSet( (QFont::CharSet)0 );
    channelList->setFont( font );
  }
  channelList->setFrameStyle( 51 );
  channelList->setLineWidth( 2 );
  channelList->setMultiSelection ( false );
  channelList->setAllColumnsShowFocus( true );
  channelList->setTreeStepSize( 0 ); 
  channelList->setRootIsDecorated( false );
  channelList->setSorting( 0 , true );
  channelList->setSorting( 1 , true );
  //channelList->setColumnAlignment ( 0, 2 ) ;

  channelList->addColumn("Nr.  ");
  channelList->addColumn("Name              ");

  //lsbutton= new QButton ( this, 0 );
  lsbutton= new QPushButton ( this, 0 );
  lsbutton->setGeometry( 165, 5, 30, 30 );
  lsbutton->setPixmap (ls_xpm);
  //  lsbutton->setToggleButton (true);

  mixerSlider_left = new winSlider( this, "Slider_left" );
  mixerSlider_left->setGeometry( 155, 40, 20, 120 );
  mixerSlider_left->setMinimumSize( 10, 10 );
  mixerSlider_left->setMaximumSize( 32767, 32767 );
  mixerSlider_left->setRange( 0, MIXER_MAX_VALUE );
  mixerSlider_left->setSteps( 1, 10 );
  mixerSlider_left->setValue( MIXER_MAX_VALUE );
  mixerSlider_left->setTracking( TRUE );
  mixerSlider_left->setTickmarks( QSlider::Left );
  mixerSlider_left->setTickInterval( 5 );

  mixerSlider_right = new winSlider( this, "Slider_right" );
  mixerSlider_right->setGeometry( 175, 40, 20, 120 );
  mixerSlider_right->setMinimumSize( 10, 10 );
  mixerSlider_right->setMaximumSize( 32767, 32767 );
  mixerSlider_right->setRange( 0, MIXER_MAX_VALUE );
  mixerSlider_right->setSteps( 1, 10 );
  mixerSlider_right->setValue( MIXER_MAX_VALUE );
  mixerSlider_right->setTracking( TRUE );
  mixerSlider_right->setTickmarks( QSlider::Right );
  mixerSlider_right->setTickInterval( 5 );

  QToolTip::add(mixerSlider_right,"set volume right");
  QToolTip::add(mixerSlider_left,"set volume left");
  QToolTip::add(channelList,"programs, klick & hold left mouse button to drag a program");
  QToolTip::add(lsbutton,"mute/unmute sound");
  

#ifdef chanDEBUG
  debug("win: init done, connecting");
#endif

  connect(mixerSlider_right,SIGNAL(valueChanged(int)),this,SLOT(rightSliderSLOT(int)));
  connect(mixerSlider_left,SIGNAL(valueChanged(int)),this,SLOT(leftSliderSLOT(int)));  
  connect(channelList,SIGNAL(selectionChanged ( QListViewItem * ) ),this,SLOT(highlightedSLOT(QListViewItem *)));
  connect(channelList,SIGNAL( doubleClicked ( QListViewItem * ) ),this,SLOT(channelEditSelectSLOT(QListViewItem *)));
  connect(channelList,SIGNAL( returnPressed ( QListViewItem * ) ),this,SLOT(channelEditSelectSLOT(QListViewItem *)));
  connect(lsbutton,SIGNAL(toggled( bool )),this,SLOT(muteSLOT(bool)));

  connect(channelList,SIGNAL(moveItemSIGNAL( QListViewItem *, QListViewItem * )),
	  this, SLOT(moveItemSLOT( QListViewItem *, QListViewItem * )));

  if (sliderAtt) {
    mixerSlider_right->hide();
    mixerSlider_left->setTickmarks( QSlider::Both );
    QToolTip::add(mixerSlider_left,"set volume");
  }

  volume_left=0;
  volume_right=0;

  fixedSliders=true;

  channelList->setFocus();
}

// =============================================================================== 

winWidget::~winWidget()
{

}

// =============================================================================== 

void winWidget::setValue(int vl, int vr)
{
#ifdef chanDEBUG
  debug("win: setValue %d, %d",vl,vr);
#endif
  if ((vl != volume_left) || (vr != volume_right)) {
    volume_left=vl;
    volume_right=vr;
    if (sliderAtt) volume_right=volume_left;
    mixerSlider_left->setValue(MIXER_MAX_VALUE-volume_left);
    mixerSlider_right->setValue(MIXER_MAX_VALUE-volume_right);
  }
}

// =============================================================================== 

void winWidget::leftSliderSLOT(int value)
{
#ifdef chanDEBUG
  //debug("volumeSLOT %d",value);
#endif
  if (volume_left != MIXER_MAX_VALUE-value) {
    volume_left=MIXER_MAX_VALUE-value;
    if (sliderAtt && (volume_left != volume_right)) 
      mixerSlider_right->setValue(MIXER_MAX_VALUE-volume_left);

    mixerSlider_left->setValue(MIXER_MAX_VALUE-volume_left);
    
    emit valueChanged(volume_left,volume_right);
  }
}

// =============================================================================== 

void winWidget::rightSliderSLOT(int value)
{
#ifdef chanDEBUG
  //debug("volumeSLOT %d",value);
#endif
  if (volume_right != MIXER_MAX_VALUE-value) {
    volume_right=MIXER_MAX_VALUE-value;
    if (sliderAtt && (volume_left != volume_right)) 
      mixerSlider_left->setValue(MIXER_MAX_VALUE-volume_right);
    
    mixerSlider_right->setValue(MIXER_MAX_VALUE-volume_right);
    
    emit valueChanged(volume_left,volume_right);
  }
}

// =============================================================================== 

void winWidget::splitSliderSLOT(int split)
{
#ifdef chanDEBUG
  debug("win: splitSliderSLOT %s",split?"unsplip":"split");
#endif

  sliderAtt=split;

  if ( sliderAtt && (volume_left != volume_right) ) {
    mixerSlider_right->setValue(MIXER_MAX_VALUE-volume_left);
  }

  if ( sliderAtt ) {
    mixerSlider_right->hide();
    mixerSlider_left->setTickmarks( QSlider::Both );
    QToolTip::add(mixerSlider_left,"set volume");
  } else {
    mixerSlider_right->show();
    mixerSlider_left->setTickmarks( QSlider::Left );
    QToolTip::add(mixerSlider_left,"set volume left");
  }

  resizeEvent(NULL);

}

// =============================================================================== 

void winWidget::channelEditSelectSLOT(QListViewItem *item)
{
#ifdef chanDEBUG
  debug("win:  channelEditSelectSLOT");
#endif
  emit channelEditSelectSIGNAL( aChannelList.find( item ) );
}

// =============================================================================== 

void winWidget::highlightedSLOT(QListViewItem * item)
{
  if (inserting) return;

  int id= aChannelList.find( item );
  
#ifdef chanDEBUG
  debug("win:  highlightedSLOT %d, %s",id,item->text(2));
#endif

  emit highlighted(id);
}

// =============================================================================== 

void winWidget::insertItem ( const channel* info, bool ins)  
{
  inserting=ins;

  QString id;
  id.sprintf("%2d",info->id);

  QString freq;
  freq.setNum(info->frequency);

  //QListViewItem *lvi= new QListViewItem( channelList, sortStr, (const char*)id, info->name, (const char*)freq );
  myQListViewItem *lvi= new myQListViewItem( channelList, (const char*)id, info->name );
  aChannelList.append(lvi);

  channelList->setSorting( 0 , true );

  inserting=false;

#ifdef chanDEBUG
  //debug("win:  insertItem %s, %s, %s, %s, %d",(const char*)id,info->name,(const char*)freq,info->channelId,inserting);
#endif
}


// =============================================================================== 

void winWidget::removeItem ( int index ) 
{
#ifdef chanDEBUG
  debug("win: removeItem");
#endif
  QListViewItem *lvi= aChannelList.at( index );
  aChannelList.remove();
  channelList->setSelected ( aChannelList.current(), true );
  channelList->setCurrentItem ( aChannelList.current() ); 
  channelList->ensureItemVisible(channelList->currentItem());
  delete lvi;
  channelList->setSorting( 0 , true );
}

// =============================================================================== 

void winWidget::changeItem ( const char * text, int index ) 
{
  QListViewItem *lvi= aChannelList.at( index );
  lvi->setText( 2, text );
  channelList->repaintItem( lvi );
}

// =============================================================================== 

void winWidget::setCurrentItem ( int index )  
{
#ifdef chanDEBUG
  debug("win: setCurrentItem %d",index);
#endif
  channelList->setSelected ( aChannelList.at( index ), true );
  channelList->setCurrentItem ( aChannelList.at( index ) );  
}

// =============================================================================== 

void winWidget::centerCurrentItem ()  
{
  channelList->ensureItemVisible( aChannelList.current() ); 
}

// =============================================================================== 

void winWidget::clear()  
{
#ifdef chanDEBUG
  debug("win: clear");
#endif

  inserting=true;

  channelList->clear();
  if ( !aChannelList.isEmpty() ) aChannelList.clear();
  channelList->setSorting( 0 , true );

  inserting=false;
}

// ================================"=============================================== 

void winWidget::muteSLOT(bool on)  
{
#ifdef chanDEBUG
  debug("win: muteSLOT %s",on?"on":"off");
#endif
}

// ================================"=============================================== 

void winWidget::moveItemSLOT( QListViewItem * fromItem, QListViewItem *toItem)
{
#ifdef chanDEBUG
  debug("win: move item %s to pos %s",fromItem->text(1),toItem->text(0));
#endif
  if ( !aChannelList.isEmpty() ) {
    QString from( fromItem->text(0) );
    QString to( toItem->text(0));

    emit moveItemSIGNAL( from.toInt(), to.toInt() ); 
  }
  
}

// =============================================================================== 

void winWidget::resizeEvent( QResizeEvent * re)
{
#ifdef chanDEBUG
  debug("win: resizeEvent");
#endif
  if (fixedSliders) {
    if (!sliderAtt) {
      int sw=40;
      channelList->setGeometry( 5, 5, width()-sw-5, height()-10 );
      mixerSlider_left->setGeometry( width()-sw+5, 40, sw/2-5, height()-10-35 );
      mixerSlider_right->setGeometry( width()-sw+(sw/2), 40, sw/2-5, height()-10-35 );
      lsbutton->setGeometry( width()-sw+5, 5 , sw - 10 , 30 );
    } else {
      int sw=35;
      channelList->setGeometry( 5, 5, width()-sw-10, height()-10 );
      mixerSlider_left->setGeometry( width()-sw, 40, sw-5 , height()-10-35 );
      lsbutton->setGeometry( width()-sw, 5 , sw - 5 , 30 );
    }
  } else {
    if (!sliderAtt) {
      int sw=width()/5;
      channelList->setGeometry( 5, 5, width()-sw-5, height()-10 );
      mixerSlider_left->setGeometry( width()-sw+5, 40, sw/2-5, height()-10-35 );
      mixerSlider_right->setGeometry( width()-sw+(sw/2), 40, sw/2-5, height()-10-35 );
      lsbutton->setGeometry( width()-sw+5, 5 , sw - 10 , 30 );
    } else {
      int sw=width()/5;
      channelList->setGeometry( 5, 5, width()-sw-10, height()-10 );
      mixerSlider_left->setGeometry( width()-sw, 40, sw-5 , height()-10-35 );
      lsbutton->setGeometry( width()-sw, 5 , sw - 5 , 30 );
    }
  }
}

// =============================================================================== 
// =============================================================================== 
// =============================================================================== 

void winSlider::mouseReleaseEvent ( QMouseEvent * me)
{
 if (me->button() == RightButton) {
#ifdef chanDEBUG
   debug("winSlider: mouseReleaseEvent");
#endif
   if (sPopup == 0) warning("winSlider: no popup");
   else sPopup->popup(mapToGlobal(me->pos()));
  }
}

// =============================================================================== 
// =============================================================================== 
// =============================================================================== 

void winListBox::rightButtonClicked ( QListViewItem *, const QPoint & pos, int) 
{
#ifdef chanDEBUG
    debug("channelListBox: mouseReleaseEvent, RightButton");
#endif
    if (cPopup == 0) warning("channelListBox: no popup");
    else cPopup->popup(mapToGlobal(pos));
}

// =============================================================================== 
#ifdef DRAGDROP
void winListBox::mouseReleaseEvent ( QMouseEvent * me )
{
  if ( !pressedButton ) QListView::mouseReleaseEvent( me );

  switch (me->button()) {
  case LeftButton:
#ifdef chanDEBUG
    debug("channelListBox: mouseReleaseEvent, LeftButton released");
#endif
    if ( itemWasMoved ) {
#ifdef chanDEBUG
      debug("channelListBox: mouseReleaseEvent, item moved, LeftButton released");
#endif
      QListViewItem *targetItem= currentItem();
      if ( targetItem && (toMoveItem != targetItem) ) {
#ifdef chanDEBUG
	debug("channelListBox: mouseReleaseEvent, moving item to %s",(const char *)targetItem->text(1));
#endif
	emit moveItemSIGNAL( toMoveItem, targetItem );
      } else {
	setCurrentItem( currentItem() );
      }
      itemWasMoved = false;
      pressedButton= 0;
    } else {
#ifdef chanDEBUG
      debug("channelListBox: mouseReleaseEvent, item not moved, LeftButton released");
#endif
      itemWasMoved = false;
      pressedButton= 0;
    }
  } // switch

}
#endif
// =============================================================================== 
#ifdef DRAGDROP
void winListBox::mousePressEvent ( QMouseEvent * me )
{
  QListView::mousePressEvent ( me );
  
  if (me->button() == LeftButton) {
#ifdef chanDEBUG
    debug("winListBox: mousePressEvent, LeftButton");
#endif
    pressedButton = LeftButton;
    toMoveItem= itemAt( me->pos() );
  } else 
    pressedButton = 0;
  
}
#endif
// =============================================================================== 

void winListBox::mouseMoveEvent ( QMouseEvent * me )
{
#ifdef DRAGDROP
  //if ( !pressedButton ) QListView::mouseMoveEvent( me );

  if (pressedButton == LeftButton) {

#ifdef chanDEBUG
    //debug("winListBox: mouseMoveEvent, LeftButton");
#endif

    oldY= me->y();

    if ( geometry().contains( me->pos() ) && (itemAt( me->pos() ) != NULL) ) {
      lastVisItem=itemAt( me->pos() );
      setCurrentItem( lastVisItem );
      /*
      ensureItemVisible( lastVisItem );

      if ( itemAt( me->pos() ) != lastVisItem ) {
	QPoint pos( mapToGlobal(me->pos()).x() , itemPos( lastVisItem ));
	QCursor::setPos( mapFromGlobal(pos) );
      }
      */
      itemWasMoved=true;
      if ( !cursorChanged ) {
	// change mouse cursor
	cursorChanged= true;
      }
#ifdef chanDEBUG
      //debug("winListBox: mouseMoveEvent: move");
#endif
    } else {
#ifdef chanDEBUG
      debug("winListBox: mouseMoveEvent: exit");
#endif
      itemWasMoved=false;
      setCurrentItem( toMoveItem );
      //ensureItemVisible ( currentItem() );
      pressedButton=0;
      cursorChanged= false;
    }
  }
#endif
}

// =============================================================================== 
// =============================================================================== 
// =============================================================================== 
// never change the next line!
#include "wintvchannel.moc"

// end of file


